@@ -18,7 +18,7 @@ This module provides factories for tuples (request, publication).
 __docformat__ = 'restructuredtext'
 
 from zope import component
-from zope.interface import implements
+from zope.interface import implementer
 from zope.app.publication.interfaces import IRequestPublicationFactory
 from zope.app.publication import interfaces
 from zope.app.publication.soap import SOAPPublication
@@ -29,10 +29,9 @@ from zope.app.publication.browser import BrowserPublication
 from zope.publisher.http import HTTPRequest
 from zope.publisher.browser import BrowserRequest
 
+@implementer(IRequestPublicationFactory)
 class SOAPFactory(object):
 
-    implements(IRequestPublicationFactory)
-
     def canHandle(self, environment):
         self.soap_req = component.queryUtility(interfaces.ISOAPRequestFactory)
         return bool(environment.get('HTTP_SOAPACTION') and self.soap_req)
@@ -41,10 +40,9 @@ class SOAPFactory(object):
         return self.soap_req, SOAPPublication
 
 
+@implementer(IRequestPublicationFactory)
 class XMLRPCFactory(object):
 
-    implements(IRequestPublicationFactory)
-
     def canHandle(self, environment):
         return True
 
@@ -54,10 +52,9 @@ class XMLRPCFactory(object):
         return request_class, XMLRPCPublication
 
 
+@implementer(IRequestPublicationFactory)
 class HTTPFactory(object):
 
-    implements(IRequestPublicationFactory)
-
     def canHandle(self, environment):
         return True
 
@@ -66,10 +63,9 @@ class HTTPFactory(object):
             interfaces.IHTTPRequestFactory, default=HTTPRequest)
         return request_class, HTTPPublication
 
+@implementer(IRequestPublicationFactory)
 class BrowserFactory(object):
 
-    implements(IRequestPublicationFactory)
-
     def canHandle(self, environment):
         return True
 
