@@ -14,10 +14,11 @@
 """A registry for Request-Publication factories.
 """
 __docformat__ = 'restructuredtext'
-from zope.interface import implements
+from zope.interface import implementer
 from zope.app.publication.interfaces import IRequestPublicationRegistry
 from zope.configuration.exceptions import ConfigurationError
 
+@implementer(IRequestPublicationRegistry)
 class RequestPublicationRegistry(object):
     """The registry implements a three stage lookup for registered factories
     that have to deal with requests::
@@ -32,7 +33,6 @@ class RequestPublicationRegistry(object):
     The `priority` is used to define a lookup-order when multiple factories
     are registered for the same method and mime-type.
     """
-    implements(IRequestPublicationRegistry)
 
     def __init__(self):
         self._d = {}   # method -> { mimetype -> {factories_data}}
@@ -41,9 +41,9 @@ class RequestPublicationRegistry(object):
         """Register a factory for method+mimetype """
 
         # initialize the two-level deep nested datastructure if necessary
-        if not self._d.has_key(method):
+        if method not in self._d:
             self._d[method] = {}
-        if not self._d[method].has_key(mimetype):
+        if mimetype not in self._d[method]:
             self._d[method][mimetype] = []
         l = self._d[method][mimetype]
 
@@ -58,7 +58,7 @@ class RequestPublicationRegistry(object):
         l.append({'name' : name, 'factory' : factory, 'priority' : priority})
 
         # order by descending priority
-        l.sort(lambda x,y: -cmp(x['priority'], y['priority']))
+        l.sort(key=lambda v: v['priority'], reverse=True)
 
         # check if the priorities are unique
         priorities = [item['priority'] for item in l]
@@ -83,27 +83,25 @@ class RequestPublicationRegistry(object):
 
 
     def lookup(self, method, mimetype, environment):
-        """Lookup a factory for a given method+mimetype and a environment."""
-
-        for m,mt in ((method, mimetype), (method, '*'), ('*', '*')):
-            factory_lst = self.getFactoriesFor(m, mt)
-            if factory_lst:
-                break
-        else:
-            raise ConfigurationError('No registered publisher found '
-                                     'for (%s/%s)' % (method, mimetype))
-
-        # now iterate over all factory candidates and let them introspect
-        # the request environment to figure out if they can handle the
-        # request
-        for d in factory_lst:
-            factory = d['factory']
-            if factory.canHandle(environment):
-                return factory
-
-        # Actually we should never get here unless of improper
-        # configuration (no default handler for method=* and mimetype=*)
-        return None
+        """Lookup a factory for a given method+mimetype and a environment.
+        """
+        factories = []
+        for m, mt in ((method, mimetype), (method, '*'), ('*', '*')):
+            # Collect, in order, from most specific to most generic, the
+            # factories that potentially handle the given environment.
+            # Now iterate over all factory candidates and let them
+            # introspect the request environment to figure out if
+            # they can handle the request. The first one to accept the
+            # environment, wins.
+            found = self.getFactoriesFor(m, mt)
+            if found is None:
+                continue
+            for info in found:
+                factory = info['factory']
+                if factory.canHandle(environment):
+                    return factory
+        raise ConfigurationError(
+            'No registered publisher found for (%s/%s)' % (method, mimetype))
 
 
 factoryRegistry = RequestPublicationRegistry()
