@@ -15,15 +15,16 @@ __docformat__ = 'restructuredtext'
 
 import sys
 import logging
-from new import instancemethod
+from types import MethodType
 
 import transaction
 
+import six
 import zope.component
 import zope.component.interfaces
 from zope.component import queryMultiAdapter
 from zope.event import notify
-from zope.interface import implements, providedBy
+from zope.interface import implementer, providedBy
 from zope.publisher.publish import mapply
 from zope.publisher.interfaces import IExceptionSideEffects, IHeld
 from zope.publisher.interfaces import IPublication, IPublishTraverse, IRequest
@@ -48,9 +49,9 @@ from zope.authentication.interfaces import IFallbackUnauthenticatedPrincipal
 from zope.authentication.interfaces import IAuthentication
 
 
+@implementer(IHeld)
 class Cleanup(object):
 
-    implements(IHeld)
 
     def __init__(self, f):
         self._f = f
@@ -66,9 +67,9 @@ class Cleanup(object):
             self._f()
 
 
+@implementer(IPublication)
 class ZopePublication(object):
     """Base Zope publication specification."""
-    implements(IPublication)
 
     root_name = 'Application'
 
@@ -204,6 +205,12 @@ class ZopePublication(object):
     def callObject(self, request, ob):
         return mapply(ob, request.getPositionalArguments(), request)
 
+    def callErrorView(self, request, view):
+        # We don't want to pass positional arguments.  The positional
+        # arguments were meant for the published object, not an exception
+        # view.
+        return mapply(view, (), request)
+
     def afterCall(self, request, ob):
         txn = transaction.get()
         if txn.isDoomed():
@@ -224,12 +231,13 @@ class ZopePublication(object):
         it's specific to this particular implementation.
         """
         if request.principal is not None:
-            txn.setUser(request.principal.id)
+            principal_id = six.text_type(request.principal.id)
+            txn.setUser(principal_id)
 
         # Work around methods that are usually used for views
         bare = removeSecurityProxy(ob)
-        if isinstance(bare, instancemethod):
-            ob = bare.im_self
+        if isinstance(bare, MethodType):
+            ob = bare.__self__
 
         # set the location path
         path = None
@@ -258,7 +266,7 @@ class ZopePublication(object):
     def _logErrorWithErrorReportingUtility(self, object, request, exc_info):
         # Record the error with the ErrorReportingUtility
         self.beginErrorHandlingTransaction(request, object,
-                                           'error reporting utility')
+                                           u'error reporting utility')
         try:
             errUtility = zope.component.getUtility(IErrorReportingUtility)
 
@@ -336,7 +344,7 @@ class ZopePublication(object):
             # We definitely have an Exception
             # Set the request body, and abort the current transaction.
             self.beginErrorHandlingTransaction(
-                request, object, 'application error-handling')
+                request, object, u'application error-handling')
             view = None
             try:
                 # We need to get a location, because some template content of
@@ -372,11 +380,7 @@ class ZopePublication(object):
 
             if view is not None:
                 try:
-                    # We use mapply instead of self.callObject here
-                    # because we don't want to pass positional
-                    # arguments.  The positional arguments were meant
-                    # for the published object, not an exception view.
-                    body = mapply(view, (), request)
+                    body = self.callErrorView(request, view)
                     response.setResult(body)
                     transaction.commit()
                     if (ISystemErrorView.providedBy(view)
@@ -386,7 +390,7 @@ class ZopePublication(object):
                         # Lame hack to get around logging missfeature
                         # that is fixed in Python 2.4
                         try:
-                            raise exc_info[0], exc_info[1], exc_info[2]
+                            six.reraise(exc_info[0], exc_info[1], exc_info[2])
                         except:
                             logging.getLogger('SiteError').exception(
                                 str(request.URL),
@@ -422,7 +426,7 @@ class ZopePublication(object):
 
             if adapter is not None:
                 self.beginErrorHandlingTransaction(
-                    request, object, 'application error-handling side-effect')
+                    request, object, u'application error-handling side-effect')
                 try:
                     # Although request is passed in here, it should be
                     # considered read-only.
